# sample_one.py

import sys
import os
import platform
import time
import wx
import wx.adv
import wx.lib.fancytext as fancytext
try:
    from wx.lib.mswalpha import draw_alpha
except ImportError:
    from mswalpha import draw_alpha
import rectshapedbitmapbuttonTwo as SBBTwo

# class MyAboutDlg
# class MyFirstFrame
# class MySecondFrame
# class MyApp

#-------------------------------------------------------------------------------

class MyAboutDlg(wx.Dialog):
    """
    ...
    """
    def __init__(self, parent):
        style = (wx.FRAME_SHAPED | wx.SIMPLE_BORDER |
                 wx.CLIP_CHILDREN | wx.STAY_ON_TOP |
                 wx.SYSTEM_MENU | wx.CLOSE_BOX |
                 wx.NO_FULL_REPAINT_ON_RESIZE)
        wx.Dialog.__init__(self,
                          parent,
                          id=-1,
                          title="About...",
                          style=style)

        #------------

        # Attributes.
        self.SetTransparent(0)
        self.opacity_in = 0
        self.opacity_out = 255
        self.deltaN = -70
        self.hasShape = False
        self.delta = wx.Point(0,0)

        #------------

        # Return application name.
        self.app_name = wx.GetApp().GetAppName()
        # Return bitmaps folder.
        self.bitmaps_dir = wx.GetApp().GetBitmapsDir()
        # Return icons folder.
        self.icons_dir = wx.GetApp().GetIconsDir()

        #------------

        # Simplified init method.
        self.SetProperties()
        self.OnTimerIn(self)
        self.CreateCtrls()
        self.BindEvents()

        #------------

        self.CenterOnParent(wx.BOTH)

        #------------

        self.ShowModal()
        self.Destroy()

    #---------------------------------------------------------------------------

    def SetProperties(self):
        """
        Set the dialog properties (title, icon, transparency...).
        """

        frameIcon = wx.Icon(os.path.join(self.icons_dir,
                                         "icon_wx.ico"),
                            type=wx.BITMAP_TYPE_ICO)
        self.SetIcon(frameIcon)


    def OnTimerIn(self, evt):
        """
        Thanks to Pascal Faut.
        """

        self.timer1 = wx.Timer(self, -1)
        self.timer1.Start(1)
        self.Bind(wx.EVT_TIMER, self.AlphaCycle1, self.timer1)

        print("Fade-in was launched.")


    def OnTimerOut(self, evt):
        """
        Thanks to Pascal Faut.
        """

        self.timer2 = wx.Timer(self, -1)
        self.timer2.Start(1)
        self.Bind(wx.EVT_TIMER, self.AlphaCycle2, self.timer2)

        print("Fade-out was launched.")


    def AlphaCycle1(self, *args):
        """
        Thanks to Pascal Faut.
        """

        self.opacity_in += self.deltaN
        if self.opacity_in <= 0:
            self.deltaN = -self.deltaN
            self.opacity_in = 0

        if self.opacity_in >= 255:
            self.deltaN = -self.deltaN
            self.opacity_in = 255

            self.timer1.Stop()

        self.SetTransparent(self.opacity_in)

        print("Fade in = {}/255".format(self.opacity_in))


    def AlphaCycle2(self, *args):
        """
        Thanks to Pascal Faut.
        """

        self.opacity_out += self.deltaN
        if self.opacity_out >= 255:
            self.deltaN = -self.deltaN
            self.opacity_out = 255

        if self.opacity_out <= 0:
            self.deltaN = -self.deltaN
            self.opacity_out = 0

            self.timer2.Stop()

            wx.CallAfter(self.Destroy)

        self.SetTransparent(self.opacity_out)

        print("Fade out = {}/255".format(self.opacity_out))

        
    def CreateCtrls(self):
        """
        Make widgets for my dialog.
        """

        # Load a background bitmap.
        self.bmp = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                          "skin_about_2.png"),
                             type=wx.BITMAP_TYPE_PNG)
        mask = wx.Mask(self.bmp, wx.RED)
        self.bmp.SetMask(mask)

        #------------

        self.SetClientSize((self.bmp.GetWidth(), self.bmp.GetHeight()))

        #------------

        if wx.Platform == "__WXGTK__":
            # wxGTK requires that the window be created before you can
            # set its shape, so delay the call to SetWindowShape until
            # this event.
            self.Bind(wx.EVT_WINDOW_CREATE, self.SetWindowShape)
        else:
            # On wxMSW and wxMac the window has already
            # been created, so go for it.
            self.SetWindowShape()


    def BindEvents(self):
        """
        Bind all the events related to my dialog.
        """

        # Bind some events to an events handler.
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_RIGHT_UP, self.OnCloseWindow)  # Panel right clic.
        self.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClick)  # View mask.
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        self.Bind(wx.EVT_WINDOW_CREATE, self.SetWindowShape)
        self.Bind(wx.EVT_MOTION, self.OnMouseMove)
        self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)


    def SetWindowShape(self, event=None):
        """
        ...
        """

        # Use the bitmap's mask to determine the region.
        r = wx.Region(self.bmp)
        self.hasShape = self.SetShape(r)


    def OnEraseBackground(self, event):
        """
        ...
        """

        dc = event.GetDC()
        if not dc:
            dc = wx.ClientDC(self)
            rect = self.GetUpdateRegion().GetBox()
            dc.SetClippingRect(rect)


    def OnDoubleClick(self, event):
        """
        ...
        """

        if self.hasShape:
            self.SetShape(wx.Region())
            self.hasShape = False
        else:
            self.SetWindowShape()


    def OnLeftDown(self, event):
        """
        ...
        """

        self.CaptureMouse()
        x, y = self.ClientToScreen(event.GetPosition())
        originx, originy = self.GetPosition()
        dx = x - originx
        dy = y - originy
        self.delta = ((dx, dy))


    def OnLeftUp(self, evt):
        """
        ...
        """

        if self.HasCapture():
            self.ReleaseMouse()


    def OnMouseMove(self, event):
        """
        ...
        """

        if event.Dragging() and event.LeftIsDown():
            x, y = self.ClientToScreen(event.GetPosition())
            fp = (x - self.delta[0], y - self.delta[1])
            self.Move(fp)


    def OnPaint(self, event):
        """
        ...
        """

        dc = wx.AutoBufferedPaintDCFactory(self)
        dc.DrawBitmap(self.bmp, -1, -1, True)

        #------------

        # These are strings.
        py_version = sys.version.split()[0]

        str1 = (('<font style="normal" family="default" color="yellow" size="10" weight="bold">'
                 'Programming : </font>'
                 '<font style="normal" family="default" color="black" size="10" weight="normal">'
                 'Python {}</font>').format(py_version))   # Python 3.6.5

        str2 = (('<font style="normal" family="default" color="red" size="10" weight="bold">'
                 'GUI toolkit : </font>'
                 '<font style="normal" family="default" color="black" size="10" weight="normal">'
                 'wxPython {}</font>').format(wx.VERSION_STRING))  # wxPython 4.0.2

        str3 = (('<font style="normal" family="default" color="brown" size="10" weight="bold">'
                 'Library : </font>'
                 '<font style="normal" family="default" color="black" size="10" weight="normal">'
                 '{}</font>').format(wx.GetLibraryVersionInfo().VersionString))   # wxWidgets 3.0.5

        str4 = (('<font style="normal" family="default" color="blue" size="10" weight="bold">'
                 'Operating system : </font>'
                 '<font style="normal" family="default" color="black" size="10" weight="normal">'
                 '{}</font>').format(platform.system()))   # Windows

        str5 = (('<font style="normal" family="default" color="gray" size="9" weight="normal">'
                 '{}</font>').format(self.app_name))   # Custom Gui 1

        str6 = (('<font style="normal" family="default" color="black" size="8" weight="normal">'
                 'Right clic or Esc for Exit</font>'))

        #------------

        # Return main image size.
        bw, bh = self.bmp.GetWidth(), self.bmp.GetHeight()

        # Draw text.
        # Need width to calculate x position of str1.
        tw, th = fancytext.GetExtent(str1, dc)
        # Centered text.
        fancytext.RenderToDC(str1, dc, (bw-tw)/2, 20)

        #------------

        # Need width to calculate x position of str2.
        tw, th = fancytext.GetExtent(str2, dc)
        # Centered text.
        fancytext.RenderToDC(str2, dc, (bw-tw)/2, 40)

        #------------

        # Need width to calculate x position of str3.
        tw, th = fancytext.GetExtent(str3, dc)
        # Centered text.
        fancytext.RenderToDC(str3, dc, (bw-tw)/2, 60)

        #------------

        # Need width to calculate x position of str4.
        tw, th = fancytext.GetExtent(str4, dc)
        # Centered text.
        fancytext.RenderToDC(str4, dc, (bw-tw)/2, 90)

        #------------

        # Need width to calculate x position of str5.
        tw, th = fancytext.GetExtent(str5, dc)
        # Centered text.
        fancytext.RenderToDC(str5, dc, (bw-tw)/2, 120)

        #------------

        # Need width to calculate x position of str6.
        tw, th = fancytext.GetExtent(str6, dc)
        # Centered text.
        fancytext.RenderToDC(str6, dc, (bw-tw)/2, 155)


    def OnKeyUp(self, event):
        """
        ...
        """

        if event.GetKeyCode() == wx.WXK_ESCAPE:
            self.OnCloseWindow(event)

        event.Skip()


    def OnCloseWindow(self, event):
        """
        ...
        """

        #self.Destroy()
        self.OnTimerOut(self)

#-------------------------------------------------------------------------------
        
class MyFirstFrame(wx.Frame):
    """
    ...
    """
    style = (wx.CLIP_CHILDREN | wx.CLOSE_BOX |
             wx.MINIMIZE_BOX | wx.SYSTEM_MENU |
             wx.NO_BORDER | wx.FRAME_SHAPED | # wx.FRAME_NO_TASKBAR
             wx.NO_FULL_REPAINT_ON_RESIZE | wx.STAY_ON_TOP)
    def __init__(self, parent, id=wx.ID_ANY, title=wx.EmptyString,
                 pos=wx.DefaultPosition, size=wx.DefaultSize,
                 style=style,
                 name='frame'):
        wx.Frame.__init__(self, parent, id, title, pos, size, style, name)
        
        #------------
 
        # Attributes.
        self.delta = wx.Point(0,0)
     
        #------------

        # Return application name.
        self.app_name = wx.GetApp().GetAppName()
        # Return bitmaps folder.
        self.bitmaps_dir = wx.GetApp().GetBitmapsDir()
        # Return icons folder.
        self.icons_dir = wx.GetApp().GetIconsDir()

        #------------

        # Create a timer.
        self.timer = wx.Timer(self)
        self.timer.Start(1000)
        self.Bind(wx.EVT_TIMER, self.OnTimer, self.timer)

        #------------

        # Simplified init method.
        self.SetProperties()
        self.CreateCtrls()
        self.BindEvents()

        #------------

        self.CenterOnScreen(wx.BOTH)
        
    #---------------------------------------------------------------------------
      
    def SetProperties(self):
        """
        ...
        """

        self.SetTitle(self.app_name)
        self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)

        frameIcon = wx.Icon(os.path.join(self.icons_dir,
                                         "icon_wx.ico"),
                            type=wx.BITMAP_TYPE_ICO)
        self.SetIcon(frameIcon)


    def CreateCtrls(self):
        """
        ...
        """
        
        # Load a background bitmap.
        self.bitmap = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                             "clouds.png"),
                                type=wx.BITMAP_TYPE_PNG)
        # or
        # image = wx.Image('phone.png', wx.BITMAP_TYPE_PNG)
        # blurimage = image.Blur(1)
        # self.bitmap = blurimage.ConvertToBitmap()

        self.SetClientSize((self.bitmap.GetWidth(), self.bitmap.GetHeight()))

        #------------
        
        if wx.Platform == "__WXGTK__":
            # wxGTK requires that the window be created before you can
            # set its shape, so delay the call to SetWindowShape until
            # this event.
            self.Bind(wx.EVT_WINDOW_CREATE, self.SetWindowShape)
        else:
            # On wxMSW and wxMac the window has
            # already been created, so go for it.
            self.SetWindowShape()

        #------------

        tt = wx.ToolTip('LeftClick to Drag - RightClick to Close')
        self.SetToolTip(tt)

        #------------
        
        # It appears that we cannot make child widgets on the Frame
        # when using mswalpha, there fore we will make a "child" shaped
        # frame that will handle widgets transparency also.
        # self.button = wx.Button(self, -1, 'Button', pos=(100, 100))

        #------------
        
        self.Centre()        
        self.widgetsframe = MySecondFrame(self,
                                          pos=self.GetPosition(),
                                          size=self.GetSize())
        self.widgetsframe.Show()

        #------------
            
        # Create an accelerator table.
        self.accel_tbl = wx.AcceleratorTable([(wx.ACCEL_CTRL, ord('X'),
                                               wx.ID_EXIT),
                                              (wx.ACCEL_CTRL, ord('A'),
                                               wx.ID_ABOUT)
                                             ])
        self.SetAcceleratorTable(self.accel_tbl)

        
    def BindEvents(self):
        """
        Bind some events to an events handle.
        """

        self.Bind(wx.EVT_MENU, self.OnAbout, id=wx.ID_ABOUT)
        self.Bind(wx.EVT_MENU, self.OnExit, id=wx.ID_EXIT)
        
        self.Bind(wx.EVT_ERASE_BACKGROUND, lambda x: None)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_MOTION, self.OnMouseMove)
        self.Bind(wx.EVT_RIGHT_UP, self.OnExit)
        self.Bind(wx.EVT_PAINT, self.OnPaint)

        
    def SetWindowShape(self, *event):
        """
        ...
        """
        
        # Use the bitmap's mask to determine the region.
        self.region = wx.Region(self.bitmap)
        self.hasShape = self.SetShape(self.region)

            
    def OnLeftDown(self, event):
        """
        ...
        """
        
        if self.HasCapture():
            self.ReleaseMouse()

        self.CaptureMouse()
        x, y = self.ClientToScreen(event.GetPosition())
        originx, originy = self.GetPosition()
        dx = x - originx
        dy = y - originy
        self.delta = (dx, dy)


    def OnLeftUp(self, event):
        """
        ...
        """
        
        if self.HasCapture():
            self.ReleaseMouse()


    def OnMouseMove(self, event):
        """
        ...
        """
        
        if event.Dragging() and event.LeftIsDown():
            x, y = self.ClientToScreen(event.GetPosition())
            fp = (x - self.delta[0], y - self.delta[1])
            self.SetPosition(fp)
            self.widgetsframe.SetPosition(fp)


    def Draw(self):
        """
        ...
        """

        # Return client size.
        width, height = self.GetClientSize()

        # Return main image size.
        bw, bh = self.bitmap.GetWidth(), self.bitmap.GetHeight()
        
        #------------

        dc = wx.MemoryDC()

        #------------
        
        fontSize = self.GetFont().GetPointSize()

        # wx.Font(pointSize, family, style, weight, underline, faceName)
        if wx.Platform == "__WXGTK__":
            self.normalBoldFont = wx.Font(fontSize-1,
                                          wx.DEFAULT, wx.NORMAL,
                                          wx.NORMAL, False, "")
            self.normalFont = wx.Font(fontSize-2,
                                      wx.DEFAULT, wx.NORMAL,
                                      wx.NORMAL, False, "")

        else:
            self.normalBoldFont = wx.Font(fontSize+23,
                                          wx.DEFAULT, wx.NORMAL,
                                          wx.NORMAL, False, "")
            self.normalFont = wx.Font(fontSize+1,
                                      wx.DEFAULT, wx.NORMAL,
                                      wx.NORMAL, False, "")

        dc.SetFont(self.normalFont)
        dc.SetFont(self.normalBoldFont)

        #------------

        bmp = wx.Bitmap.FromRGBA(width, height, red=0, green=0, blue=0, alpha=0)
        dc.SelectObject(bmp)

        #------------
        
        gc = wx.GraphicsContext.Create(dc)
        gcdc = wx.GCDC(gc)

        # Draw a bitmap with an alpha channel
        # on top of the last group.
        # image, x, y, transparence
        gcdc.DrawBitmap(self.bitmap, -1, -1, useMask=False)

        # Draw text.
        gcdc.SetTextForeground(wx.Colour(250, 250, 250, 255))  # grey
        gcdc.SetTextBackground(wx.TransparentColour)
        gcdc.SetFont(self.normalFont)
        date = time.strftime("%A %d %B %Y")
        tw, th = gcdc.GetTextExtent(date)
        gcdc.DrawText(date, (int((bw-tw)/2), int(133)))

        gcdc.SetTextForeground(wx.Colour(95, 95, 95, 255))  # white
        gcdc.SetTextBackground(wx.TransparentColour)
        gcdc.SetFont(self.normalBoldFont)
        hour = time.strftime("%H:%M:%S")  # "%I:%M:%S %p"
        tw, th = gcdc.GetTextExtent(hour)
        gcdc.DrawText(hour, (int((bw-tw)/2-5), int(90)))

        #------------

        gcdc.Destroy()
        del gcdc

        dc.Destroy()
        del dc

        #------------

        draw_alpha(self, bmp)

        
    def OnTimer(self, event):
        """
        ...
        """

        dc = wx.BufferedDC(wx.ClientDC(self))
        dc.Clear()
        
        self.Draw()
        
        
    def OnPaint(self, event):
        """
        ...
        """

        dc = wx.BufferedPaintDC(self)
        dc.Clear()
        
        self.Draw()
  

    def OnAbout(self, event):
        """
        ...
        """

        self.dialog = MyAboutDlg(self)

        
    def OnExit(self, event):
        """
        ...
        """

        self.widgetsframe.Close()
        self.Close()
        
#-------------------------------------------------------------------------------
        
class MySecondFrame(wx.Frame):
    """
    ...
    """
    def __init__(self, parent, id=wx.ID_ANY, title=wx.EmptyString,
                 pos=wx.DefaultPosition, size=wx.DefaultSize,
                 style=wx.FRAME_SHAPED |
                       wx.NO_BORDER |
                       wx.FRAME_NO_TASKBAR |
                       wx.FRAME_FLOAT_ON_PARENT,
                 name="frame"):
        wx.Frame.__init__(self, parent, id, title, pos, size, style, name)

        #------------

        # Attributes.
        self.SetTransparent(255)  
        
        #------------

        # Return bitmaps folder.
        self.bitmaps_dir = wx.GetApp().GetBitmapsDir()
        # Return icons folder.
        self.icons_dir = wx.GetApp().GetIconsDir()

        #------------

        # Simplified init method.
        self.SetProperties()
        self.CreateCtrls()
        self.BindEvents()

    #---------------------------------------------------------------------------

    def SetProperties(self):
        """
        ...
        """

        self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)
        
        
    def CreateCtrls(self):
        """
        ...
        """

        self.slider = wx.Slider(self, -1,
                                value=160,
                                minValue=64,
                                maxValue=255,
                                size=(0, 0),
                                pos=(195, 20),
                                style=wx.SL_HORIZONTAL)
        self.slider.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)
        
        #------------
        
        # Button About.
        self.btn1 = SBBTwo.ShapedBitmapButton(self, -1,
            bitmap=wx.Bitmap(os.path.join(self.bitmaps_dir,
                                          "btn_about_normal1.png"),
                             type=wx.BITMAP_TYPE_PNG),

            pressedBmp=wx.Bitmap(os.path.join(self.bitmaps_dir,
                                              "btn_about_pressed1.png"),
                                 type=wx.BITMAP_TYPE_PNG),

            hoverBmp=wx.Bitmap(os.path.join(self.bitmaps_dir,
                                            "btn_about_normal1.png"),
                               type=wx.BITMAP_TYPE_PNG),

            disabledBmp=wx.Bitmap(os.path.join(self.bitmaps_dir,
                                               "btn_about_disabled1.png"),
                                  type=wx.BITMAP_TYPE_PNG),

            label="",
            labelForeColour=wx.WHITE,
            labelFont=wx.Font(9,
                              wx.FONTFAMILY_DEFAULT,
                              wx.FONTSTYLE_NORMAL,
                              wx.FONTWEIGHT_BOLD),
            style=wx.BORDER_NONE)
        
        self.btn1.SetPosition((int((169)), int(19)))
        self.btn1.SetSize((15, 15))

        #------------
        
        if wx.Platform == "__WXGTK__":
            # wxGTK requires that the window be created before you can
            # set its shape, so delay the call to SetWindowShape until
            # this event.
            self.Bind(wx.EVT_WINDOW_CREATE, self.SetWindowShape)
        else:
            # On wxMSW and wxMac the window has
            # already been created, so go for it.
            self.SetWindowShape()


    def BindEvents(self):
        """
        Bind some events to an events handler.
        """
       
        self.slider.Bind(wx.EVT_SLIDER, self.OnSlider)        
        self.btn1.Bind(wx.EVT_BUTTON, self.OnBtnAbout)       

        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        
      
    def SetWindowShape(self, *event):
        """
        ...
        """
        
        # Use the widget's rect's to determine the region.
        wxRegion = wx.Region
        self.region = wxRegion()
        widgets = [child for child in self.GetChildren()]
        regions = [wxRegion(widget.GetRect()) for widget in widgets]
        Union = self.region.Union
        [Union(reg) for reg in regions]
        self.hasShape = self.SetShape(self.region)
           

    def OnLeftDown(self, event):
        """
        ...
        """
        
        if self.HasCapture():
            self.ReleaseMouse()

        self.CaptureMouse()
        x, y = self.ClientToScreen(event.GetPosition())
        originx, originy = self.GetPosition()
        dx = x - originx
        dy = y - originy
        self.delta = (dx, dy)


    def OnLeftUp(self, event):
        """
        ...
        """
        
        if self.HasCapture():
            self.ReleaseMouse()

            
    def OnBtnAbout(self, event):
        """
        ...
        """

        self.GetParent().OnAbout(self)


    def OnBtnClose(self, event):
        """
        ...
        """

        self.GetParent().OnExit(self)
       
        print("Close button was clicked.")


    def OnSlider(self, event):
        """
        ...
        """
        
        self.SetTransparent(event.GetEventObject().GetValue())
      
#-------------------------------------------------------------------------------

class MyApp(wx.App):
    """
    Thanks to Andrea Gavana.
    """
    def OnInit(self):

        #------------

        self.locale = wx.Locale(wx.LANGUAGE_FRENCH)

        #------------

        self.SetAppName("Custom Gui 1")

        #------------

        self.installDir = os.path.split(os.path.abspath(sys.argv[0]))[0]

        #------------

        frame = MyFirstFrame(None)
        self.SetTopWindow(frame)
        frame.Show(True)

        return True

    #---------------------------------------------------------------------------

    def GetInstallDir(self):
        """
        Returns the installation directory for my application.
        """

        return self.installDir


    def GetIconsDir(self):
        """
        Returns the icons directory for my application.
        """

        icons_dir = os.path.join(self.installDir, "icons")
        return icons_dir


    def GetBitmapsDir(self):
        """
        Returns the bitmaps directory for my application.
        """

        bitmaps_dir = os.path.join(self.installDir, "bitmaps")
        return bitmaps_dir

#-------------------------------------------------------------------------------

def main():
    app = MyApp(False)
    app.MainLoop()

#-------------------------------------------------------------------------------

if __name__ == "__main__" :
    wxVER = 'wxPython %s' % wx.version()
    pyVER = 'python %d.%d.%d.%s' % sys.version_info[0:4]
    versionInfos = '%s %s' % (wxVER, pyVER)
    print(versionInfos)
    main()

